//                              -*- Mode: C++ -*- 
// 
// uC++ Version 5.4.0, Copyright (C) Peter A. Buhr 1998
// 
// EHM2.cc -- 
// 
// Author           : Peter A. Buhr
// Created On       : Tue Oct 27 21:24:48 1998
// Last Modified By : Peter A. Buhr
// Last Modified On : Tue Sep 12 08:08:53 2006
// Update Count     : 29
// 

#include <uC++.h>

_ThrowEvent xxx {
  public:
    uBaseTask *tid;
    xxx( uBaseTask *tid ) : tid(tid) {}
};

_ThrowEvent yyy {
  public:
    uBaseTask *tid;
    yyy( uBaseTask *tid ) : tid(tid) {}
};


_Task fred {
    void r( int i ) {
	if ( i == 0 ) {
	    _Throw xxx( &uThisTask() );
	} else {
	    r( i - 1 );
	} // if
    } // fred::r

    void s( int i ) {
	if ( i == 0 ) {
	    try {
		r( 5 );
	    } catch( xxx e ) {
		assert( e.tid == &uThisTask() );
		_Throw;
	    }
	} else {
	    s( i - 1 );
	} // if
    } // fred::s

    void t( int i ) {
	if ( i == 0 ) {
	    try {
		s( 5 );
	    } catch( xxx e ) {
		assert( e.tid == &uThisTask() );
		_Throw yyy( &uThisTask() );
	    }
	} else {
	    t( i - 1 );
	} // if
    } // fred::t
    
    void main() {
//	for ( int i = 0; i < 5000; i += 1 ) {
	for ( int i = 0; i < 5; i += 1 ) {
	    try {
		t( 5 );
	    } catch( yyy e ) {
		assert( e.tid == &uThisTask() );
	    } // try
	} // for
    } // fred::main
}; // fred

void uMain::main() {
    uProcessor p[3];
    fred f[4] __attribute__(( unused ));
} // uMain::main
